@@ -2,6 +2,7 @@ package serial
 
 import (
 	"io"
+	"path/filepath"
 
 	"github.com/xtls/xray-core/core"
 	"github.com/xtls/xray-core/infra/conf"
@@ -9,9 +10,12 @@ import (
 )
 
 func BuildConfig(files []string, formats []string) (*core.Config, error) {
-
 	cf := &conf.Config{}
-	for i, file := range files {
+	for i, file_ := range files {
+		file, err := filepath.EvalSymlinks(file_)
+		if err != nil {
+			return nil, err
+		}
 		newError("Reading config: ", file).AtInfo().WriteToLog()
 		r, err := confloader.LoadConfig(file)
 		if err != nil {
@@ -32,9 +36,7 @@ func BuildConfig(files []string, formats []string) (*core.Config, error) {
 
 type readerDecoder func(io.Reader) (*conf.Config, error)
 
-var (
-	ReaderDecoderByFormat = make(map[string]readerDecoder)
-)
+var ReaderDecoderByFormat = make(map[string]readerDecoder)
 
 func init() {
 	ReaderDecoderByFormat["json"] = DecodeJSONConfig
