@@ -44,5 +44,5 @@ func executeQueryStats(cmd *base.Command, args []string) {
 	if err != nil {
 		base.Fatalf("failed to query stats: %s", err)
 	}
-	showResponese(resp)
+	showJSONResponse(resp)
 }
